unit CdRomActions;

interface

  procedure HandleCdRom(DriveLetter : String);

  { next are all private function used only in this unit
  function IsNumeric(InputString : String) : Boolean;
  procedure FileSearch(const PathLocation : string);
  function CanAutorunCD(DriveLetter : String) : Boolean;
  function ContainsMedia(DriveLetter : String) : Boolean;
  function IsVideoCD(DriveLetter : String) : Boolean;
  function IsPlaystationCD(DriveLetter : String) : Boolean;
  }

implementation

uses
  Classes, SysUtils, Dialogs, IniFile, Main, Windows, FileCtrl,
  PlaystationAltList, IniFiles;

var
  ListOfFiles : TStringList;
  CountAudio, CountVideo : Integer;

function IsNumeric(InputString : String) : Boolean;
var
  i : integer;
begin
  { initialise }
  IsNumeric := True;
  try
    i := StrToInt(InputString);
  except
    { niet gelukt, niet numeriek }
    IsNumeric := False;
  end;
end;

function IsAudioExtension(strExtension : String) : Boolean;
var
  i : Integer;
begin
  IsAudioExtension := False;
  strExtension := LowerCase(strExtension);
  strExtension := Copy(strExtension, 2, Length(strExtension));
  for i := 0 to ini_AudioExtensions.Count-1 do
    if (ini_AudioExtensions[i] = strExtension) then
    begin
      IsAudioExtension := True;
      Break;
    end;
end;

function IsVideoExtension(strExtension : String) : Boolean;
var
  i : Integer;
begin
  IsVideoExtension := False;
  strExtension := LowerCase(strExtension);
  strExtension := Copy(strExtension, 2, Length(strExtension));
  for i := 0 to ini_VideoExtensions.Count-1 do
    if (ini_VideoExtensions[i] = strExtension) then
    begin
      IsVideoExtension := True;
      Break;
    end;
end;


procedure FileSearch(const PathLocation : string);
{ search entire cd for all relevant files, put file list in ListOfFiles }
var
  SearchRec  : TSearchRec;
  strPath, strFilename, strExt : string;
  boolRelevantFile : Boolean;
begin
  strPath := IncludeTrailingBackslash(PathLocation);
  { search all files }
  if FindFirst(strPath + '*.*', faAnyFile - faDirectory, SearchRec) = 0 then
  try
    repeat
      { check extension }
      boolRelevantFile := False;
      strFilename := strPath + SearchRec.Name;
      strExt := LowerCase(ExtractFileExt(strFilename));
      { is it an audio file }
      if IsAudioExtension(strExt) then begin
        { audio file }
        boolRelevantFile := True;
        Inc(CountAudio);
      end
      else if IsVideoExtension(strExt) then begin
        { video file }
        boolRelevantFile := True;
        Inc(CountVideo);
      end;
      { add filenames to stringlist }
      if boolRelevantFile then
        ListOfFiles.Add(strFilename);
    until FindNext(SearchRec) <> 0;
  finally
    SysUtils.FindClose(SearchRec);
  end;

  { also search all subdirs, FileSearch is called recursively }
  if FindFirst(strPath + '*.*', faDirectory, SearchRec) = 0 then
  try
    repeat
    if ((SearchRec.Attr and faDirectory) <> 0)
    and (SearchRec.Name <> '.')
    and (SearchRec.Name <> '..') then
      { FileSearch is called recursively here }
      FileSearch(strPath + SearchRec.Name);
    until FindNext(SearchRec) <> 0;
  finally
    SysUtils.FindClose(SearchRec);
  end;
end;

procedure PlayAudioFiles;
var
  i : Integer;
  strFileName, strTemp : String;
  AudioList : TextFile;
begin
  { maak afspeellijst aan als er meerdere audio bestanden gevonden zijn }
  if (CountAudio > 1) then begin
    strFileName := ExtractFilePath(ini_AppName) + 'temp.m3u';
    Assign(AudioList, strFileName);
    Rewrite(AudioList);
  end;

  { loop door alle gevonden bestanden }
  for i := 0 to ListOfFiles.Count-1 do
  begin
    { kijk of extensie een extensie van een audio-bestandstype is }
    strTemp := LowerCase(ExtractFileExt(ListOfFiles[i]));
    { indien video bestand gevonden in "relevantebestanden"-lijst }
    if IsAudioExtension(strTemp) then
      if (CountAudio = 1) then
        strFileName := ListOfFiles[i]       { slechts 1 bestand }
      else
        Writeln(AudioList, ListOfFiles[i]); { bestand aan lijst toevoegen }
  end;

  { sluit afspeellijst als er meerdere audio bestanden zijn }
  if (CountAudio > 1) then
    CloseFile(AudioList);

  { start m3u with audio program }
  strFileName := '"' + strFileName + '"'; {could be spaces in name}
  frmMain.RunApplication(ini_Audio, strFileName, '');
end;

procedure PlayVideoFiles(boolVideoCD : Boolean);
var
  i : Integer;
  strFileName, strTemp : String;
  VideoList : TextFile;
begin
  { maak afspeellijst aan als er meerdere video bestanden gevonden zijn }
  if (CountVideo > 1) then begin
    strFileName := ExtractFilePath(ini_AppName) + 'temp.m3u';
    Assign(VideoList, strFileName);
    Rewrite(VideoList);
  end;

  { loop door alle relevante gevonden bestanden }
  for i := 0 to ListOfFiles.Count-1 do
  begin
    { kijk of extensie een extensie van een video-bestandstype is }
    strTemp := LowerCase(ExtractFileExt(ListOfFiles[i]));
    { indien video bestand gevonden in "relevantebestanden"-lijst }
    if IsVideoExtension(strTemp) then
      if (CountVideo = 1) then
        strFileName := ListOfFiles[i]       { slechts 1 bestand }
      else
        Writeln(VideoList, ListOfFiles[i]); { bestand aan lijst toevoegen }
  end;

  { sluit afspeellijst als er meerdere video bestanden zijn }
  if (CountVideo > 1) then
    CloseFile(VideoList);

  { start m3u with video program }
  strFileName := '"' + strFileName + '"'; {could be spaces in name}
  if not frmMain.RunApplication(ini_Movies, strFileName, '') then
    if boolVideoCD then
      frmMain.SetStatusInfo('Video CD program not found')
    else
      frmMain.SetStatusInfo('Movie program not found');
end;

function IsPlaystationCD(DriveLetter : String) : String;
{ te herkennen aan 2 bestanden in root:
  SYSTEM
  SL*S_###.## bijv SLUS_004.99, SLPS_018.00 }
var
  strSearchMask, strTemp : String;
  FileAttrs : Integer;
  sr : TSearchRec;
begin
  IsPlaystationCD := '';

  { playstation cd?}
  if FileExists(DriveLetter + 'SYSTEM.CNF') then
  begin
    strSearchMask := DriveLetter + 'SL*.*';
    FileAttrs := faAnyFile - faDirectory - faVolumeID;
    if FindFirst(strSearchMask, FileAttrs, sr) = 0 then
    begin
      strTemp := Copy(sr.Name, 6, 3);
      if IsNumeric(strTemp) then
      begin
        strTemp := Copy(sr.Name, 10, 2);
        if IsNumeric(strTemp) then
          IsPlaystationCD := sr.Name;
      end;
    end;
    SysUtils.FindClose(sr);
  end;
end;

function IsAlternatePlaystationCD(strCDid : String) : Boolean;
var
  i : Integer;
  strTemp : String;
begin
  IsAlternatePlaystationCD := False;
  strCDid := LowerCase(Trim(strCDid));
  { check if alternate playstation cd }
  for i := 0 to ini_PSXAltCDlist.Count-1 do
  begin
    strTemp := LowerCase(Trim(ini_PSXAltCDlist.Strings[i]));
    strTemp := Copy(strTemp, 1, 11); {for example SLPS131.13 }
    { if given CDid is in list }
    if (strTemp = strCDid) then
      IsAlternatePlaystationCD := True;
  end;
end;

function IsVideoCD(DriveLetter : String) : Boolean;
var
  i : Integer;
begin
  IsVideoCD := False;
  { must have folder called 'MPEGAV' }
  If not DirectoryExists(DriveLetter + 'MPEGAV') then Exit;
  { make backup of extensions list, and alter it }
  {clear list}
  ListOfFiles.Clear;
  CountVideo := 0;
  { temporary add '.DAT' to video extensions }
  i := ini_VideoExtensions.Add('dat');
  { search all files in MPEGAV folder }
  FileSearch(DriveLetter + '');
  { any audio or video files found? }
  If (CountVideo > 0) then
    PlayVideoFiles(True);
  { temporary add '.DAT' to video extensions }
  ini_VideoExtensions.Delete(i);
  { return result }
  IsVideoCD := (CountVideo > 0);
end;

function IsAudioCD(DriveLetter : String) : Boolean;
begin
  IsAudioCD := False;
  { must have file called 'track01.cda' }
  If not FileExists(DriveLetter + 'track01.cda') then Exit;
  { play cd with audioCD program }
  if not frmMain.RunApplication(ini_AudioCD, DriveLetter, '') then
    frmMain.SetStatusInfo('Audio CD program not found');

  { return result }
  IsAudioCD := True;
end;

function ContainsMedia(DriveLetter : String) : Boolean;
begin
  {clear list}
  ListOfFiles.Clear;
  CountAudio := 0;
  CountVideo := 0;
  { search all files, }
  FileSearch(DriveLetter);
  { any audio or video files found? }
  If (CountAudio > 0) and (CountVideo = 0) then
    PlayAudioFiles;
  If (CountAudio = 0) and (CountVideo > 0) then
    PlayVideoFiles(False);
  { return result }
  ContainsMedia := (CountAudio > 0) or (CountVideo > 0);
end;

function CanAutorunCD(DriveLetter : String) : Boolean;
var
  AutorunFile : TIniFile; {autorun heeft zelfde opbouw als .ini files }
  strCommand : String;
  i : Integer;
begin
  CanAutorunCD := False;

  { is het een autorun cd? }
  if not FileExists(DriveLetter + 'autorun.inf') then Exit;

  AutorunFile := TIniFile.Create(DriveLetter + 'autorun.inf');
  strCommand := AutorunFile.ReadString('autorun', 'open', '');
  AutorunFile.Free;
  strCommand := LowerCase(Trim(strCommand));
  if (strCommand = '') then Exit;

  if (copy(strCommand, 1, 5) = 'start') then
  begin
    { verander bijv. "start eenfilm.mpg" in "start d:\eenfilm.mpg"}
    for i := 6 to Length(strCommand) do
      if Copy(strCommand, i, 1) <> ' ' then
        break;
    Insert(DriveLetter, strCommand, i);
  end
  else
    strCommand := DriveLetter + strCommand;

  CanAutorunCD := True;

  { alleen als aangevinkt }
  if not ini_Autorun then
    Exit
  else
    { run the command of autorun file }
    if not frmMain.RunApplication(strCommand, '', DriveLetter) then
      frmMain.SetStatusInfo('Error with autorun.inf file');

end;

Procedure HandleCdRom(DriveLetter : String);
{ Do something with cd }
var
  strPlaystation : String;
  tmpByte : Byte;
  strTemp : String;
begin
  DriveLetter := DriveLetter + ':\';
  { -- playstation -- }
  strPlaystation := IsPlaystationCD(DriveLetter);
  If strPlaystation <> '' Then Begin
    { save variable }
    strLastPSXcd := strPlaystation;
    { what kind of playstation cd }
    strTemp := UpperCase(Copy(strPlaystation, 3, 1));
    tmpByte := Ord(Char(strTemp[1]));
    case tmpByte of
    {E} 69 : strTemp := '(EUR)';
    {P} 80 : strTemp := '(JAP)';
    {U} 85 : strTemp := '(USA)';
    end; {case}
    frmMain.SetStatusInfo('Playstation CD ' + strTemp);
    if IsAlternatePlaystationCD(strPlaystation) then
    begin
      { alternate playstation emulator program }
      if not frmMain.RunApplication(ini_PlaystationAlt, '', '') then
        frmMain.SetStatusInfo('Playstation alt. emulator program not found');
    end
    else
      { normal playstation emulator program }
      if not frmMain.RunApplication(ini_Playstation, '', '') then
        frmMain.SetStatusInfo('Playstation emulator program not found');
  end
  { -- video cd -- }
  Else If IsVideoCD(DriveLetter) Then Begin
    frmMain.SetStatusInfo('Video CD');
  end
  { -- audio cd -- }
  Else If IsAudioCD(DriveLetter) Then Begin
    frmMain.SetStatusInfo('Audio CD');
  end
  { movie or audio files }
  Else If ContainsMedia(DriveLetter) Then Begin
    {..}
    { both audio and video files found}
    If (CountAudio > 0) and (CountVideo > 0) then
    begin
      { ask 'play audio or video?' }
      if Windows.MessageBox(0, PChar('Both audio and video files were found.'+#13+'Do you want to play the audio files?'), PChar(ini_AppName), MB_YESNO+MB_ICONASTERISK) = ID_YES then
        PlayAudioFiles
      else if Windows.MessageBox(0, PChar('Do you want to play the video files then?'), PChar(ini_AppName), MB_YESNO+MB_ICONASTERISK) = ID_YES then
        PlayVideoFiles(False);
    end;

    { audio files were found, add to status caption }
    if (CountAudio > 0) then begin
      strTemp := strTemp + IntToStr(CountAudio) + ' audio file';
      if (CountAudio > 1) then strTemp := strTemp + 's';
    end;

    { video files were found, add to status caption }
    if (CountVideo > 0) then
    begin
      if (strTemp <> '') then
        strTemp := strTemp + ' and ';
      strTemp := strTemp + IntToStr(CountVideo) + ' video file';
      if (CountVideo > 1) then strTemp := strTemp + 's';
    end;
    frmMain.SetStatusInfo('CD-ROM with ' + strTemp);
  end
  { -- use autorun -- }
  Else If CanAutorunCD(DriveLetter) Then Begin
    frmMain.SetStatusInfo('AutoRun CD');
  end
  { -- unknown cd type -- }
  else begin
    frmMain.SetStatusInfo('Unknown CD type');
  end;

end;

initialization
  ListOfFiles := TStringList.Create();
  ListOfFiles.Clear;

finalization
  ListOfFiles.Free;

end.
