unit IniFile;

interface

uses
  Classes;

function ReadIniFile : Boolean;
function WriteIniFile : Boolean;

var
  ini_FileName, ini_AppName : String;

  { settings }
  ini_Playstation, ini_PlaystationAlt, ini_VideoCD, ini_AudioCD, ini_Movies,
  ini_Audio : String;
  ini_Autorun : Boolean;
  ini_PSXAltCDlist : TStringList;
  ini_AudioExtensions, ini_VideoExtensions : TStringList;

implementation

uses
  IniFiles, SysUtils, Windows;

function ReadIniFile : Boolean;
var
  AppIni : TIniFile;
  strMessage, strTemp, strExt : String;
  i, j : Integer;
  TempList : TStringList;
begin
  ReadIniFile := False;

  { get .ini file from parameter }
  ini_AppName := ParamStr(0);
  ini_FileName := paramstr(1);

  { if no ini file given, use applicationname + '.ini' }
  if (ini_FileName = '') then
    ini_FileName := ChangeFileExt(ini_AppName, '.ini');

  { add current path if needed }
  If (ExtractFileDir(ini_FileName) = '') then
    ini_FileName := ExtractFileDir(ini_AppName) + ini_FileName;

  AppIni := TIniFile.Create(ini_FileName);

  { settings }
  ini_Playstation := AppIni.ReadString('settings', 'playstation', 'ERROR');
  ini_PlaystationAlt := AppIni.ReadString('settings', 'playstationalt', 'ERROR');
  ini_VideoCD := AppIni.ReadString('settings', 'videocd', 'ERROR');
  ini_AudioCD := AppIni.ReadString('settings', 'audiocd', 'ERROR');
  ini_Movies := AppIni.ReadString('settings', 'movies', 'ERROR');
  ini_Audio := AppIni.ReadString('settings', 'audio', 'ERROR');
  strTemp := AppIni.ReadString('settings', 'autorun', 'Yes');
  ini_Autorun := (LowerCase(Copy(strTemp, 1, 1)) <> 'n');

  { -- alternate playstation cd -- }
  ini_PSXAltCDlist.Clear;
  TempList := TStringList.Create();
  try
    AppIni.ReadSectionValues('psxaltlist', TempList);
    for i := 0 to TempList.Count-1 do
    begin
      strTemp := Trim(TempList.Strings[i]);
      j := Pos('=', strTemp);
      if (strTemp <> '')
      and (j <> 0) then
        ini_PSXAltCDlist.Add(strTemp);
    end;
  finally
    TempList.Free;
  end; {try}

  { -- audio extensions -- }
  ini_AudioExtensions.Clear;
  strTemp := AppIni.ReadString('settings', 'audioextensions', 'mp1;mp2;mp3;wav');
  repeat
    i := Pos(';', strTemp);
    if (i = 0) then i := Length(strTemp)+1;
    strExt := LowerCase(Copy(strTemp, 1, i-1));
    Delete(strTemp, 1, i);
    ini_AudioExtensions.Add(strExt);
  until strTemp = '';
  { -- video extensions -- }
  ini_VideoExtensions.Clear;
  strTemp := AppIni.ReadString('settings', 'videoextensions', 'asf;avi;mpg;mpeg');
  repeat
    i := Pos(';', strTemp);
    if (i = 0) then i := Length(strTemp)+1;
    strExt := LowerCase(Copy(strTemp, 1, i-1));
    Delete(strTemp, 1, i);
    ini_VideoExtensions.Add(strExt);
  until strTemp = '';

  AppIni.Free;

  { if file not exists }
  if not FileExists(ini_FileName) then begin
    WriteIniFile;
    strMessage := 'Could not find the file ' + ini_FileName;
    Windows.MessageBox(0, Pchar(strMessage), Pchar('Error'), MB_OK+MB_ICONWARNING);
  end
  else
    ReadIniFile := True;

end;

function WriteIniFile : Boolean;
var
  AppIni : TiniFile;
  i, j : Integer;
  strTemp, strCDid, strDescription : String;
begin
  WriteIniFile := False;

  AppIni := TIniFile.Create(ini_Filename);

  { settings, always write }
  AppIni.WriteString('settings', 'playstation', ini_Playstation);
  AppIni.WriteString('settings', 'playstationalt', ini_PlaystationAlt);

  AppIni.WriteString('settings', 'videocd', ini_VideoCD);
  AppIni.WriteString('settings', 'audiocd', ini_AudioCD);
  AppIni.WriteString('settings', 'movies', ini_Movies);
  AppIni.WriteString('settings', 'audio', ini_Audio);
  if ini_Autorun then
    AppIni.WriteString('settings', 'autorun', 'Yes')
  else
    AppIni.WriteString('settings', 'autorun', 'No');

  { alternate playstation cd }
  for i := 0 to ini_PSXAltCDlist.Count-1 do
  begin
    strTemp := ini_PSXAltCDlist.Strings[i];
    j := Pos('=', strTemp);
    strCDid := Copy(strTemp, 1, j-1);
    strDescription := Copy(strTemp, j+1, Length(strTemp)-j);
    AppIni.WriteString('psxaltlist', strCDid, strDescription);
  end;

  { audio }
  strTemp := '';
  for i := 0 to ini_AudioExtensions.Count-1 do
    strTemp := strTemp + ini_AudioExtensions[i] + ';';
  AppIni.WriteString('settings', 'audioextensions', strTemp);
  { video }
  strTemp := '';
  for i := 0 to ini_VideoExtensions.Count-1 do
    strTemp := strTemp + ini_VideoExtensions[i] + ';';
  AppIni.WriteString('settings', 'videoextensions', strTemp);

  AppIni.Free;

  WriteIniFile := True;
end;

initialization
  ini_PSXAltCDlist := TStringList.Create();
  ini_AudioExtensions := TStringList.Create();
  ini_VideoExtensions := TStringList.Create();

finalization
  ini_PSXAltCDlist.Free;
  ini_AudioExtensions.Free;
  ini_VideoExtensions.Free;

end.

