unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ShellApi, AppEvnts, Menus;

  const
(* Events of WM_DEVICECHANGE (wParam) *)

  DBT_DEVICEARRIVAL = $8000;            (* system detected a new device *)
  DBT_DEVICEQUERYREMOVE = $8001;        (* wants to remove, may fail *)
  DBT_DEVICEQUERYREMOVEFAILED = $8002;  (* removal aborted *)
  DBT_DEVICEREMOVEPENDING = $8003;      (* about to remove, still avail *)
  DBT_DEVICEREMOVECOMPLETE = $8004;     (* device is gone *)
  DBT_DEVICETYPESPECIFIC = $8005;       (* type specific event *)
  DBT_CONFIGCHANGED = $0018;

 (* type of device in DEV_BROADCAST_HDR *)
  DBT_DEVTYP_OEM = $00000000;           (* OEM- or IHV-defined *)
  DBT_DEVTYP_DEVNODE = $00000001;       (* Devnode number *)
  DBT_DEVTYP_VOLUME = $00000002;        (* Logical volume *)
  DBT_DEVTYP_PORT = $00000003;          (* Port (serial or parallel *)
  DBT_DEVTYP_NET = $00000004;           (* Network resource *)

  (* media types in DBT_DEVTYP_VOLUME *)
  DBTF_MEDIA = $0001;                (* change affects media in drive *)
  DBTF_NET = $0002;                     (* logical volume is network volume *)

  { for showing systray popup menu }
  UM_NOTIFYICON = WM_USER + 124;
  MYTRAYICONID = 0;

type
  {The first thing we make is a message-specific record for this windows
   message. We could use TMessage, but then we have to work with the
   meaningless words like wParam and lParam.}
  TWMDeviceChange = record
    Msg : Cardinal;
    Event : UINT;
    dwData : Pointer;
    Result : LongInt;
  end;
  {When we got a DBT_DEVICECHANGE or DBT_DEVICEREMOVECOMPLETE event, dwData
  contains an address of a DEV_BROADCAST_HDR structure identifying the device
  inserted. We also need a pointer so we can point to this structure.}
  PDEV_BROADCAST_HDR = ^TDEV_BROADCAST_HDR;
  TDEV_BROADCAST_HDR = packed record
    dbch_size : DWORD;
    dbch_devicetype : DWORD;
    dbch_reserved : DWORD;
  end;

  {When the device is of type volume, then we can get some device specific
   information, namely specific information about a logical volume.}
  PDEV_BROADCAST_VOLUME = ^TDEV_BROADCAST_VOLUME;
  TDEV_BROADCAST_VOLUME = packed record
    dbcv_size : DWORD;
    dbcv_devicetype : DWORD;
    dbcv_reserved : DWORD;
    dbcv_unitmask : DWORD;
    dbcv_flags : WORD;
  end;

  TfrmMain = class(TForm)
    grpSettings: TGroupBox;
    lblPlaystation: TLabel;
    lblMovies: TLabel;
    lblAudio: TLabel;
    edtPlaystation: TEdit;
    edtMovies: TEdit;
    edtAudio: TEdit;
    grpStatus: TGroupBox;
    lblStatus: TLabel;
    lblBdR: TLabel;
    edtVideoCD: TEdit;
    lblVideoCD: TLabel;
    chkAutorun: TCheckBox;
    edtPlaystationAlt: TEdit;
    edtAudioCD: TEdit;
    lblPlaystationAlt: TLabel;
    btnPlaystationAlt: TButton;
    lblAudioCD: TLabel;
    popSystrayMenu: TPopupMenu;
    miRestore: TMenuItem;
    miClose: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure chkAutorunClick(Sender: TObject);
    procedure edtPlaystationChange(Sender: TObject);
    procedure edtVideoCDChange(Sender: TObject);
    procedure edtMoviesChange(Sender: TObject);
    procedure edtAudioChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure edtPlaystationAltChange(Sender: TObject);
    procedure edtAudioCDChange(Sender: TObject);
    procedure btnPlaystationAltClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure miRestoreClick(Sender: TObject);
    procedure miCloseClick(Sender: TObject);
  private
    { Private declarations }
    TrayIcon: TNotifyIconData; { systray icon data }
    procedure RestoreMainForm; { restore from systray  }
    procedure ShowThePopup; { systray popup menu }
    procedure UMNotifyIcon(var Msg: TMessage); message UM_NOTIFYICON; { show systray popup menu }

    procedure MinimizeToTray(Sender: TObject); { put in systray }
    procedure WMDeviceChange(var Msg: TWMDeviceChange);
    message WM_DEVICECHANGE; {we capture the WM_DEVICECHANGE message}
  public
    { Public declarations }
    function RunApplication(strApplication, strParams, strStartupDir : string) : boolean;
    procedure SetStatusInfo(Info: String);    
  end;

  function GetFirstDriveLetter(unitmask : longint) : char;

var
  frmMain: TfrmMain;

implementation

uses
  CdRomActions, IniFile, PlaystationAltList;

{$R *.DFM}
{$R WindowsXP.RES}

{ ---- systray stuff ---- }
procedure TfrmMain.MinimizeToTray(Sender: TObject);
begin
  Shell_notifyIcon(NIM_ADD, @TrayIcon);
  frmMain.Visible := false;
end;

procedure TfrmMain.RestoreMainForm;
begin
  frmMain.Visible := true;
  Application.Restore;
  Application.BringToFront;
  SetFocus; { needed for Win98 when double click systray-icon, not nescessarily needed on WinNT/XP }
  Shell_notifyIcon(NIM_DELETE, @TrayIcon);
end;

procedure TfrmMain.ShowThePopup;
var
  CurPos: TPoint;
begin
  SetForeGroundWindow (Application.Handle);
  GetCursorPos(CurPos);
  Application.ProcessMessages;
  popSystrayMenu.Popup(CurPos.x, CurPos.y);
{  PostMessage(Self.Handle, WM_NULL, 0, 0);}
end;

(*procedure TfrmMain.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  { user clicked on systray icon }
  if X = 515 then       // double click left
    RestoreMainForm
  else if X = 517 then  // single click right
    ShowThePopup;
end;*)
procedure TfrmMain.UMNotifyIcon(var Msg: TMessage);
var
  uID: integer;
  uMouseMsg: integer;
begin
  uID := Msg.wParam;
  uMouseMsg := Msg.lParam;
  if uID = MYTRAYICONID then // we may have created more than one icon
    case uMouseMsg of
      WM_LBUTTONDOWN:
        ShowThePopup;
      WM_LBUTTONDBLCLK:
        RestoreMainForm;
      WM_RBUTTONDOWN:
        ShowThePopup; //
    end;
end;

procedure TfrmMain.miRestoreClick(Sender: TObject);
begin
  RestoreMainForm;
end;

procedure TfrmMain.miCloseClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  Shell_notifyIcon(NIM_DELETE, @TrayIcon);
end;

{ ---- cd changing stuff ---- }
procedure TfrmMain.WMDeviceChange
          (var Msg: TWMDeviceChange);
var
  lpdb : PDEV_BROADCAST_HDR;
  lpdbv : PDEV_BROADCAST_VOLUME;
  strDriveLetter : String;
begin

 (* received a wm_devicechange message *)
  lpdb := PDEV_BROADCAST_HDR(Msg.dwData);
  (* look at the event send together with the wm_devicechange message *)
   case Msg.Event of
   DBT_DEVICEARRIVAL :
     begin
       if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then
       begin
         lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
         if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
{ should use fAfterArrival but I don't know how.. }
{           if Assigned(fAfterArrival) then
             fAfterArrival(Self, GetFirstDriveLetter(lpdbv^.dbcv_unitmask));}
         begin
           strDriveLetter := GetFirstDriveLetter(lpdbv^.dbcv_unitmask);
           SetStatusInfo('CD inserted in drive ' +  strDriveLetter);
           HandleCdRom(strDriveLetter);
         end;
       end;
     end;
   DBT_DEVICEREMOVECOMPLETE :
     begin
       if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then
       begin
         lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
         if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
{ should use fAfterArrival but I don't know how.. }
{          if Assigned(fAfterArrival) then
           fAfterRemove(Self, GetFirstDriveLetter(lpdbv^.dbcv_unitmask));}
           strDriveLetter := GetFirstDriveLetter(lpdbv^.dbcv_unitmask);
           SetStatusInfo('CD removed from drive ' +  strDriveLetter);
       end;
     end;
   end; {case}
end;

function GetFirstDriveLetter(unitmask : longint) : char;
{returns the drive letter contained in UnitMask}
var
  DriveLetter : shortint;
begin
  DriveLetter := Ord('A');
  while (unitmask and 1)=0  do begin
    unitmask := unitmask shr 1;
    inc(DriveLetter);
  end;
  Result := Char(DriveLetter);
end;

procedure TfrmMain.SetStatusInfo(Info: String);
begin
  lblStatus.Caption := Info;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  if not ReadIniFile then
    ShowMessage('Please check the settings.');

  { toon settings uit ini file }
  edtPlaystation.Text := ini_Playstation;
  edtPlaystationAlt.Text := ini_PlaystationAlt;
  edtVideoCD.Text := ini_VideoCD;
  edtAudioCD.Text := ini_AudioCD;
  edtMovies.Text := ini_Movies;
  edtAudio.Text := ini_Audio;
  chkAutorun.Checked := ini_Autorun;

  { systray stuff }
  TrayIcon.cbSize := SizeOf(TrayIcon);
  TrayIcon.Wnd := Self.Handle;
  TrayIcon.uID := 0;
  TrayIcon.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
  TrayIcon.uCallbackMessage := WM_MOUSEMOVE;
  TrayIcon.hIcon := Application.Icon.Handle;
  TrayIcon.szTip := 'CD starter';
  TrayIcon.uID := MYTRAYICONID; { for showing systray popupmenu }
  TrayIcon.uCallbackMessage := UM_NOTIFYICON; { for showing systray popupmenu }

  Application.OnMinimize := MinimizeToTray;
end;

function TfrmMain.RunApplication(strApplication, strParams, strStartupDir : string) : boolean;
{ start an external application with parameters }
var
  CmdLine: array[0..255] of char;
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  i : Integer;
  charStartupDir : array[0..255] of Char;
  pStartupDir : PChar;
begin
  RunApplication := False;

  if (strApplication = '') then
    { geen application gegeven }
    Exit;

  { commandline string opbouwen }
  StrPCopy(CmdLine, strApplication + ' ' + strParams);
  FillChar(StartupInfo, SizeOf(StartupInfo), 0);
  StartupInfo.cb := SizeOf(StartupInfo);
  StartupInfo.dwFlags := STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow := SW_SHOWNORMAL;
  { create string startupdir }
  pStartupDir := nil;
  if (strStartupDir <> '') then begin
    for i := 0 to length(strStartupDir) do
      charStartupDir[i] := strStartupDir[i+1];
    pStartupDir := charStartupDir;
  end;

  { create the external windows process }
  if CreateProcess(nil,CmdLine,nil,nil,false,NORMAL_PRIORITY_CLASS,
                   nil,pStartupDir,StartupInfo,ProcessInfo) then
    begin
      Result := true;
      // wait for process to end..
      WaitForSingleObject(ProcessInfo.hThread, INFINITE);
      CloseHandle(ProcessInfo.hThread);
      CloseHandle(ProcessInfo.hProcess);
    end;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  WriteIniFile;
end;

procedure TfrmMain.chkAutorunClick(Sender: TObject);
begin
  ini_Autorun := chkAutorun.Checked;
end;

procedure TfrmMain.edtPlaystationChange(Sender: TObject);
begin
  ini_Playstation := edtPlaystation.Text;
end;

procedure TfrmMain.edtPlaystationAltChange(Sender: TObject);
begin
  ini_PlaystationAlt := edtPlaystationAlt.Text;
end;

procedure TfrmMain.edtVideoCDChange(Sender: TObject);
begin
  ini_VideoCD := edtVideoCD.Text;
end;

procedure TfrmMain.edtAudioCDChange(Sender: TObject);
begin
  ini_AudioCD := edtAudioCD.Text;
end;

procedure TfrmMain.edtMoviesChange(Sender: TObject);
begin
  ini_Movies := edtMovies.Text;
end;

procedure TfrmMain.edtAudioChange(Sender: TObject);
begin
  ini_Audio := edtAudio.Text;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
begin
  HandleCdRom('D');
end;

procedure TfrmMain.btnPlaystationAltClick(Sender: TObject);
begin
  with TfrmPlaystationAltList.Create(Self) do begin
    try
      ShowModal;
    finally
      Free;
    end; {try}
  end;
end;

end.
